import {
  select,
  csv,
  scaleLinear,
  scaleBand,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const xAxisTickFormat = (number) => format('.3s')(number);

const render = (data) => {
  const xValue = (d) => d.Month;
  const xAxisLabel = 'Month';

  const yValue = (d) => d.ElectronicsSales;
  const yAxisLabel = 'Electronics Sales';

  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 150,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleBand()
    .domain(data.map(xValue))
    .range([0, innerWidth])
    .padding(-0);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -93)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 75)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  const lineGenerator = line()
    .x((d) => xScale(xValue(d)) + xScale.bandwidth() / 2)
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);

  const areaGenerator = area()
    .x((d) => xScale(xValue(d)) + xScale.bandwidth() / 2)
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis);

  g.append('path')
    .attr('class', 'line-chart')
    .attr('d', lineGenerator(data));

  g.append('path')
    .attr('class', 'area-chart')
    .attr('d', areaGenerator(data));
};

csv(
  'https://gist.githubusercontent.com/Akshitha-Rapala/2a6804121ef1c8aaafc98541e935f327/raw/c26cca755b92c66c3cab552fb5e255a3652ce9d2/ElectronicsSales.csv',
).then((data) => {
  data.forEach((d) => {
    d.ElectronicsSales = +d.ElectronicsSales;
    // Removed the Date conversion for Month
  });
  render(data);
  console.log(data);
});
